//
// This code has been put together (and re-coded by myself, John White) using the lcc-win32
// example codes from Drive Jump and CD Door.
//
// The code creates a system tray with multiple menu items, if need be, to represent your
// installed CD Drives. The idea being that you can open and/or close a particular CD Door.
// The code could be changed to recognize all 26 different drives (Floppy, HD and Floppy).
// So you could have a menu that shows disk info for A: and B: (like DJ.c), allow a CD Door
// to be Opened/Closed, show info about C:\, etc. In other words, this is an ideal shell
// (basic/template) code for further enhancements.
//
// John White - yoingco@hotmail.com   http://www.btinternet.com/~yoingco/yoingco/
//
// Link with: shell32.lib  winmm.lib
//

#include <windows.h>
#include <windowsx.h>
#include <winuser.h>
#include <commctrl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <shellapi.h>
#include <mmsystem.h>

#define IDAPPLICON 710

HINSTANCE handle_Instance;
HWND wnd0Ptr;
NOTIFYICONDATA nid;
HMENU menu;

long rtn=0;

LRESULT CALLBACK wnd0Procedure(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

BOOL InitApplication(void);
HWND Create_Window(void);
void create_traymenu(void);
void create_tray(void);
void cd_door(BOOL,BOOL,char *);

BOOL InitApplication(void)
{
	WNDCLASS wc0;
	memset(&wc0,0,sizeof(WNDCLASS));
	wc0.style=CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS|CS_BYTEALIGNCLIENT|CS_BYTEALIGNWINDOW;
	wc0.lpfnWndProc=(WNDPROC)wnd0Procedure;
	wc0.hInstance=handle_Instance;
	wc0.hbrBackground=(HBRUSH)(COLOR_WINDOW+1);
	wc0.lpszClassName="JWW0";
	wc0.lpszMenuName=NULL;
	wc0.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc0.hIcon=LoadIcon(handle_Instance,MAKEINTRESOURCE(IDAPPLICON));
	if (!RegisterClass(&wc0)) {
		return 0;
	}
	return 1;
}

HWND Create_Window()
{
	return CreateWindowEx(WS_EX_TOOLWINDOW,"JWW0","",WS_POPUP,CW_USEDEFAULT,
				0,CW_USEDEFAULT,0,NULL,NULL,handle_Instance,NULL);
}

LRESULT CALLBACK wnd0Procedure(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	HDC hdc;
	POINT coords;
	PAINTSTRUCT ps;
	RECT rect;
	HANDLE sbrush;
	COLORREF colour;
	WORD menuitem=0;
	char stgbuf[10];
	char letter[]="Z";

	switch (msg) {
	case WM_PAINT:
		colour=GetSysColor(COLOR_MENU);
		hdc=BeginPaint(wnd0Ptr,&ps);
		GetClientRect(wnd0Ptr,&rect);
		SetBkMode(hdc,OPAQUE);
		rtn=SetBkColor(hdc,colour);
		sbrush=CreateSolidBrush(colour);
		FillRect(hdc,&rect,sbrush);
		DeleteObject(sbrush);
		EndPaint(wnd0Ptr,&ps);
		return 0;

	case WM_USER:
		switch (lParam)	{
		case WM_LBUTTONUP:
			ShowWindow(wnd0Ptr,SW_SHOW);
			SetForegroundWindow(wnd0Ptr);
			break;

		case WM_RBUTTONUP:
			GetCursorPos(&coords);
			if (menu) {
				TrackPopupMenu(menu,TPM_RIGHTALIGN|TPM_RIGHTBUTTON,
						coords.x,coords.y,0,wnd0Ptr,NULL);
			}
			break;
		}
		break;

	case WM_COMMAND:
		menuitem=LOWORD(wParam);
		if (menuitem==3053) {
			EnableWindow(wnd0Ptr,FALSE);
			ShowWindow(wnd0Ptr,SW_HIDE);
			PostQuitMessage(0);
			break;
		}

		if ((menuitem>2999) && (menuitem<3052)) {
			if ((menuitem>2999) && (menuitem<3026)) {
				letter[0]=(char)((65+(menuitem-3000)));
			} else {
				letter[0]=(char)((65+(menuitem-3026)));
			}
			lstrcpy(stgbuf,letter);
			lstrcat(stgbuf,":");
			if ((menuitem>2999) && (menuitem<3026)) {
				cd_door(0,1,stgbuf);
			} else {
				cd_door(1,1,stgbuf);
			}
		}
		break;

	case WM_SHOWWINDOW:
		if (wParam==TRUE) {
		}
		break;

	case WM_DESTROY:
		EnableWindow(wnd0Ptr,FALSE);
		ShowWindow(wnd0Ptr,SW_HIDE);
		PostQuitMessage(0);
		return 0;

	default:
		return DefWindowProc(hwnd,msg,wParam,lParam);
	}
	return 0;
}

void create_traymenu(void)
{
	MENUITEMINFO mi[54];
	DWORD drives;
	char counter=0,count=0;
	char drive[]="Z:\\";
	char letter[]="Z";
	char stgbuf[20];
	mi[52].cbSize=sizeof(MENUITEMINFO);
	mi[52].fMask=MIIM_ID|MIIM_STATE|MIIM_TYPE;
	mi[52].fType=MFT_SEPARATOR;
	mi[52].fState=MFS_ENABLED;
	mi[52].wID=3052;
	mi[52].hSubMenu=NULL;
	mi[52].hbmpChecked=NULL;
	mi[52].hbmpUnchecked=NULL;
	mi[52].dwItemData=0;
	mi[52].dwTypeData=0;
	mi[52].cch=0;
	mi[53].cbSize=sizeof(MENUITEMINFO);
	mi[53].fMask=MIIM_ID|MIIM_STATE|MIIM_TYPE;
	mi[53].fType=MFT_STRING;
	mi[53].fState=MFS_DEFAULT|MFS_ENABLED;
	mi[53].wID=3053;
	mi[53].hSubMenu=NULL;
	mi[53].hbmpChecked=NULL;
	mi[53].hbmpUnchecked=NULL;
	mi[53].dwItemData=0;
	mi[53].dwTypeData="Quit";
	mi[53].cch=4;
	menu=CreatePopupMenu();
	drives=GetLogicalDrives();
	for (count=0;count<26;count++) {
		if (drives & 1) {
			drive[0]=(char)(65+count);
			rtn=GetDriveType(drive);
			if (rtn==DRIVE_CDROM) {      // if (rtn>1) {   <-- use something like this for all drives.
				letter[0]=(char)(65+count);
				mi[counter].cbSize=sizeof(MENUITEMINFO);
				mi[counter].fMask=MIIM_ID|MIIM_STATE|MIIM_TYPE;
				mi[counter].fType=MFT_STRING;
				mi[counter].fState=MFS_ENABLED;
				mi[counter].wID=count+3000;
				mi[counter].hSubMenu=NULL;
				mi[counter].hbmpChecked=NULL;
				mi[counter].hbmpUnchecked=NULL;
				mi[counter].dwItemData=0;
				lstrcpy(stgbuf,letter);
				lstrcat(stgbuf," - Open Door");
				mi[counter].dwTypeData=stgbuf;
				mi[counter].cch=13;
				InsertMenuItem(menu,counter,TRUE,&mi[counter]);
				mi[counter+1].cbSize=sizeof(MENUITEMINFO);
				mi[counter+1].fMask=MIIM_ID|MIIM_STATE|MIIM_TYPE;
				mi[counter+1].fType=MFT_STRING;
				mi[counter+1].fState=MFS_ENABLED;
				mi[counter+1].wID=count+3026;
				mi[counter+1].hSubMenu=NULL;
				mi[counter+1].hbmpChecked=NULL;
				mi[counter+1].hbmpUnchecked=NULL;
				mi[counter+1].dwItemData=0;
				lstrcpy(stgbuf,letter);
				lstrcat(stgbuf," - Close Door");
				mi[counter+1].dwTypeData=stgbuf;
				mi[counter+1].cch=14;
				InsertMenuItem(menu,(counter+1),TRUE,&mi[counter+1]);
				counter=counter+2;

				drives=drives>>1;
				InsertMenuItem(menu,counter,TRUE,&mi[52]);
				InsertMenuItem(menu,(counter+1),TRUE,&mi[53]);
			}
		}
	}
}

void create_tray(void)
{
	nid.cbSize=sizeof(NOTIFYICONDATA);
	nid.hWnd=wnd0Ptr;
	nid.uID=23;
	nid.uFlags=NIF_ICON|NIF_TIP|NIF_MESSAGE;
	nid.uCallbackMessage=WM_USER;
	nid.hIcon=LoadIcon(handle_Instance,MAKEINTRESOURCE(IDAPPLICON));
	lstrcpy (nid.szTip,"Open/Close A Particular CD Door");
	Shell_NotifyIcon(NIM_ADD,&nid);
}

void cd_door(BOOL ocflag,BOOL use_defdrive,char *driveletter)
{
	MCI_OPEN_PARMS op;
	DWORD flags;
	ZeroMemory(&op,sizeof(MCI_OPEN_PARMS));
	op.lpstrDeviceType=(LPCSTR)MCI_DEVTYPE_CD_AUDIO;
	if (use_defdrive) {
		flags=MCI_OPEN_TYPE|MCI_OPEN_TYPE_ID|MCI_OPEN_SHAREABLE|MCI_OPEN_ELEMENT;
		op.lpstrElementName=driveletter;
	} else {
		flags=MCI_OPEN_TYPE|MCI_OPEN_TYPE_ID|MCI_OPEN_SHAREABLE;
	}
	rtn=mciSendCommand(0,MCI_OPEN,flags,(unsigned long)&op);
	if (rtn==0) {
		if (ocflag) {
			mciSendCommand(op.wDeviceID,MCI_SET,MCI_SET_DOOR_CLOSED,0);
		} else {
			mciSendCommand(op.wDeviceID,MCI_SET,MCI_SET_DOOR_OPEN,0);
		}
		mciSendCommand(op.wDeviceID,MCI_CLOSE,MCI_WAIT,0);
	} else {
	}
}

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
							LPSTR lpCmdLine,INT nCmdShow)
{
	MSG msg;
	HANDLE mutex_handle;
	handle_Instance=hInstance;
	mutex_handle=CreateMutex(NULL,TRUE,"OCCDDD");
	if (!mutex_handle){
		MessageBox(NULL,"CD MenuTray V1.00 is already running!",
			"  CD MenuTray V1.00",
			MB_OK|MB_ICONINFORMATION|MB_SYSTEMMODAL);
		return 1;
	}

	if (!InitApplication())
		goto cleanup;

	if (!(wnd0Ptr=Create_Window()))
		goto cleanup;

	create_traymenu();
	create_tray();
	while (GetMessage(&msg,NULL,0,0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

cleanup:
	Shell_NotifyIcon(NIM_DELETE,&nid);
	if (menu) DestroyMenu(menu);
	if (mutex_handle) ReleaseMutex(mutex_handle);

	return msg.wParam;

}


